from uuid import uuid4
import uuid
from django.conf import settings
from event.serializers import EventEditSerializer, EventSerializer
from rest_framework.views import APIView
from rest_framework.permissions import IsAuthenticated
from rest_framework.parsers import MultiPartParser, FormParser
from rest_framework.response import Response
from rest_framework import status
from .models import Booking, Event, Bank
from django.contrib.auth.models import User
from rest_framework.generics import ListAPIView
from django.shortcuts import get_object_or_404
from rest_framework.exceptions import ValidationError
from django.core.mail import send_mail
from rest_framework.exceptions import PermissionDenied, ValidationError
from django.utils.timezone import now
from django.db.models import Q
from social_app.models import Follow

class BankListView(APIView):
    def get(self, request):
        banks = Bank.objects.all().values('id', 'name')
        return Response({"banks": list(banks)}, status=status.HTTP_200_OK)

class CreateEventView(APIView):
    permission_classes = [IsAuthenticated]
    parser_classes = [MultiPartParser, FormParser]

    def post(self, request):
        data = request.data
        print(data)

        try:
            # Validate and fetch bank
            bank_id = data.get('bank_id')
            try:
                bank = Bank.objects.get(id=bank_id)
            except Bank.DoesNotExist:
                return Response({"error": "Invalid bank ID"}, status=status.HTTP_400_BAD_REQUEST)

            # Create event
            event = Event.objects.create(
                organisation_type=data.get('organisation_type'),
                organisation_name=data.get('organisation_name'),
                full_name=data.get('full_name'),
                mobile_number=data.get('mobile_number'),
                event_type=data.get('event_type'),
                pan_number=data.get('pan_number'),
                address=data.get('address'),
                gstin_status=data.get('gstin_status'),
                gst_number=data.get('gst_number'),
                email=data.get('email'),
                bank=bank,
                beneficiary_name=data.get('beneficiary_name'),
                account_number=data.get('account_number'),
                ifsc_code=data.get('ifsc_code'),
                title=data.get('title'),
                overview=data.get('overview'),
                location=data.get('location'),
                amount=data.get('amount'),
                total_tickets=data.get('total_tickets'),
                available_tickets=data.get('total_tickets'),
                event_category=data.get('event_category', None),
                event_date=data.get('event_date'),
                event_time=data.get('event_time'),
                upload_pan=request.FILES.get('upload_pan'),
                upload_cancelled_cheque=request.FILES.get('upload_cancelled_cheque'),
                thumbnail=request.FILES.get('thumbnail'),
                created_by=request.user
            )

            # Success Response
            response_data = {
                "organisation_type": event.organisation_type,
                "pan_number": event.pan_number,
                "address": event.address,
                "gstin_status": event.gstin_status,
                "gst_number": event.gst_number,
                "fullname": event.beneficiary_name,
                "user_id": event.created_by.id,
                "email": event.email,
                "bank_details": {
                    "bank_name": bank.name,
                    "beneficiary_name": event.beneficiary_name,
                    "account_number": event.account_number,
                    "ifsc_code": event.ifsc_code,
                },
                "event_details": {
                    "title": event.title,
                    "overview": event.overview,
                    "location": event.location,
                    "amount": str(event.amount),
                    "total_tickets": event.total_tickets,
                    "event_category": event.event_category,
                    "event_date": event.event_date,
                    "event_time": event.event_time,
                },
                "uploads": {
                    "pan_file": event.upload_pan.name,
                    "cancelled_cheque": event.upload_cancelled_cheque.name,
                },
            }
            return Response({"success":True,"message": "Event added successfully", "event": response_data}, status=status.HTTP_201_CREATED)

        except Exception as e:
            return Response({"error": str(e)}, status=status.HTTP_500_INTERNAL_SERVER_ERROR)

class EventListView(ListAPIView):

    queryset = Event.objects.all()
    serializer_class = EventSerializer
    permission_classes = [IsAuthenticated]
    def list(self, request, *args, **kwargs):
        try:

            events = self.get_queryset()
            serializer = self.get_serializer(events, many=True)
            return Response({
                "status": "success",
                "data": serializer.data
            }, status=status.HTTP_200_OK)

        except Exception as e:
            return Response({
                "status": "error",
                "message": str(e)
            }, status=status.HTTP_500_INTERNAL_SERVER_ERROR)

class EventDetailView(APIView):
    def get(self, request):
        event_id = request.query_params.get('event_id')
        print(event_id)
        event = get_object_or_404(Event, id=event_id)
        return Response({
            "message": "Event details fetched successfully",
            "event": {
                "id": event.id,
                "address": event.address,
                "organisation_type":event.organisation_type,
                "organisation_name":event.organisation_name,
                "full_name":event.full_name,
                "mobile_number":event.mobile_number,
                "event_type":event.event_type,
                "created_user": event.created_by.get_full_name(),
                "user_id": event.created_by.id,
                "event_thumbnail": event.thumbnail.url if event.thumbnail else None,
                "email": event.email,
                "event_details": {
                    "title": event.title,
                    "overview": event.overview,
                    "location": event.location,
                    "amount": event.amount,
                    "available_tickets": event.available_tickets,
                    "event_category": event.event_category,
                    "event_date": event.event_date,
                    "event_time": event.event_time,
                },
            },
        })

class EventBookingView(APIView):
    permission_classes = [IsAuthenticated]

    def post(self, request, event_id):
        event = get_object_or_404(Event, id=event_id)
        user = request.user
        if event.created_by == user:
            return Response({"message": "You cannot book tickets for your own event."}, status=403)

        # Get data from request body
        ticket_count = request.data.get('ticket_count')
        amount_paid = request.data.get('amount_paid')
        transaction_id = request.data.get('transaction_id')
        payment_status = request.data.get('payment_status')

        # Validate inputs
        if not ticket_count or not amount_paid or not payment_status:
            raise ValidationError("Missing required fields.")
        if event.available_tickets == 0:
            return Response({"message": "All tickets are sold"}, status=400)
        if event.available_tickets < int(ticket_count):
            return Response({"message": "Not enough tickets available."}, status=400)
        if event.status==0:
            return Response({"message": "Event is inactive you can't book ticket"}, status=400)

        # Generate a unique booking ID
        booking_id = f"evn{uuid.uuid4().hex[:8]}"

        # Create booking record
        booking = Booking.objects.create(
            event=event,
            user=user,
            ticket_count=ticket_count,
            amount_paid=amount_paid,
            transaction_id=transaction_id,
            payment_status=payment_status,
            booking_id=booking_id,
        )

        # Reduce available tickets
        event.available_tickets=event.available_tickets - int(ticket_count)
        event.save()
        print(payment_status)
        if payment_status.lower() == "success":
            try:
                send_mail(
                    subject="Event Booking Confirmation",
                    message=(
                        f"Dear {user.first_name},\n\n"
                        f"Thank you for booking tickets for '{event.title}'.\n\n"
                        f"Booking Details:\n"
                        f"Booking ID: {booking_id}\n"
                        f"Event: {event.title}\n"
                        f"Tickets: {ticket_count}\n"
                        f"Amount Paid: ₹{amount_paid}\n\n"
                        f"We look forward to seeing you at the event!\n\n"
                        f"Best Regards,\n"
                        f"Event Team" #Organizer name
                    ),
                    from_email=settings.EMAIL_HOST_USER,
                    recipient_list=[user.mobile_or_email],
                    fail_silently=False,
                )
            except Exception as e:
                return Response({"message": "Event booked but failed to send confirmation email.", "error": str(e)}, status=500)

        # Prepare response
        response_data = {
            "message": "Event booked successfully",
            "transaction_id": transaction_id,
            "payment_status": payment_status,
            "event_details": {
                "title": event.title,
                "ticket_count": ticket_count,
                "amount_paid": amount_paid,
                "Booking_id": booking_id,
                "Qr_code": booking.qr_code.url,
            }
        }
        return Response(response_data, status=201)
    
class VerifyPaymentView(APIView):
    permission_classes = [IsAuthenticated]

    def post(self, request, event_id):
        transaction_id = request.data.get('transaction_id')

        if not transaction_id:
            return Response(
                {"message": "Transaction ID is required"},
                status=status.HTTP_400_BAD_REQUEST,
            )
        event = get_object_or_404(Event, id=event_id)

        try:
            booking = Booking.objects.get(event=event, transaction_id=transaction_id)
        except Booking.DoesNotExist:
            return Response(
                {"message": "Payment failed or pending", "payment_status": "Failure", "transaction_id": transaction_id},
                status=status.HTTP_404_NOT_FOUND,
            )

        # Build the success response
        if booking.payment_status == "success":
            return Response(
                {
                    "message": "Payment confirmed",
                    "payment_status": "Success",
                    "transaction_id": transaction_id,
                    "event_details": {
                        "title": event.title,
                        "total_tickets": booking.ticket_count,
                        "amount": booking.amount_paid,
                    },
                },
                status=status.HTTP_200_OK,
            )
        else:
            return Response(
                {"message": "Payment failed or pending", "payment_status": booking.payment_status, "transaction_id": transaction_id},
                status=status.HTTP_400_BAD_REQUEST,
            )
        
class EventAppliedUsersView(APIView):

    def get(self, request, event_id):
        try:
            # Filter the bookings for a specified event where payment is successful
            bookings = Booking.objects.filter(event_id=event_id, payment_status='success')

            if not bookings.exists():
                return Response(
                    {"message": "No tickets were purchased for this event."},
                    status=status.HTTP_404_NOT_FOUND
                )

            # Construct the response data
            applied_users = [
                {
                    "user_id": booking.user.id,
                    "username": booking.user.username,
                    "email": booking.user.email,
                    "ticket_count": booking.ticket_count,
                    "amount_paid": float(booking.amount_paid),
                    "booking_id": booking.booking_id,
                    "transaction_id": booking.transaction_id,
                    "booking_date": booking.created_at.strftime("%Y-%m-%d %H:%M:%S")
                }
                for booking in bookings
            ]

            return Response(
                {"event_id": event_id, "applied_users": applied_users},
                status=status.HTTP_200_OK
            )

        except Booking.DoesNotExist:
            return Response(
                {"error": "Event or booking data not found"},
                status=status.HTTP_404_NOT_FOUND
            )
        except Exception as e:
            return Response(
                {"error": str(e)},
                status=status.HTTP_500_INTERNAL_SERVER_ERROR
            )

class UpdateEventStatus(APIView):
    def patch(self, request, event_id):
        # Retrieve the event by ID
        event = get_object_or_404(Event, id=event_id)
        
        # Get the status from the request data
        new_status = request.data.get("status")
        if new_status not in [0, 1]:
            return Response(
                {"error": "Invalid status value. Use 1 for active and 0 for inactive."},
                status=status.HTTP_400_BAD_REQUEST,
            )

        # Update the event's status
        event.status = new_status
        event.save()

        return Response(
            {"message": f"Event status updated to {'active' if new_status == 1 else 'inactive'}.",
             "event_id": event.id,
             "new_status": event.status},
            status=status.HTTP_200_OK,
        )
    
class MyEventsView(APIView):
    permission_classes = [IsAuthenticated]

    def get(self, request):
        user = request.user
        my_events = Event.objects.filter(created_by=user).order_by('-event_date')  # Fetch events created by the user
        serializer = EventSerializer(my_events, many=True)
        return Response({
            "status": "success",
            "data": serializer.data
        }, status=200)

class EditEventView(APIView):
    permission_classes = [IsAuthenticated]

    def patch(self, request, event_id):
        user = request.user
        event = get_object_or_404(Event, id=event_id)
        if event.created_by != user:
            raise PermissionDenied("You do not have permission to edit this event.")

        allowed_fields = {'thumbnail', 'available_tickets', 'overview'}
        data = {key: value for key, value in request.data.items() if key in allowed_fields}

        if not data:
            raise ValidationError("Only 'thumbnail', 'available_tickets', and 'overview' can be updated.")
        serializer = EventEditSerializer(event, data=data, partial=True)
        if serializer.is_valid():
            serializer.save()
            return Response({"status": "success", "message": "Event details updated successfully."}, status=200)
        else:
            return Response(serializer.errors, status=400)
        
class UpcomingEventsView(APIView):
    permission_classes = [IsAuthenticated]

    def get(self, request, *args, **kwargs):
        user = request.user

        # Get industry names selected by the user
        selected_industry_names = user.profile.selected_industries.values_list('name', flat=True)

        following_users = Follow.objects.filter(follower=user).values_list('following', flat=True)

        followers = Follow.objects.filter(following=user).values_list('follower', flat=True)

        # Combine following and followers
        related_users = set(following_users).union(set(followers))

        # Filter upcoming events based on selected industries and related users
        events = Event.objects.filter(
            Q(event_category__in=selected_industry_names) | Q(created_by__in=related_users),
            event_date__gte=now().date(),
            status=1  # Assuming 1 indicates the event is active
        ).order_by('event_date', 'event_time')

        # Serialize the events
        serializer = EventSerializer(events, many=True, context={'request': request})

        if serializer.data:
            return Response({
                "success": True,
                "message": "Upcoming events retrieved successfully.",
                "data": serializer.data
            }, status=status.HTTP_200_OK)
        else:
            return Response({
                "success": False,
                "message": "No upcoming events found.",
            }, status=status.HTTP_404_NOT_FOUND)
        
class EventListAPI(APIView):
    def get(self, request, *args, **kwargs):
        event = Event.objects.all()
        serializer = EventSerializer(event,many=True, context={"request": request})
        return Response({"sucess": True, "data":serializer.data}, status=200)
    
class ShareEventView(APIView):
    def post(self, request):
        event_id = request.data.get('event_id')
        if not event_id:
            return Response({"error": "Event ID is required."}, status=status.HTTP_400_BAD_REQUEST)
        event = get_object_or_404(Event, id=event_id)

        # Generate the event detail URL
        base_url = request.build_absolute_uri('/')
        event_detail_url = f"{base_url}api/event/details/?event_id={event_id}"

        return Response(
            {"message": "Shareable URL generated successfully.", "url": event_detail_url},
            status=status.HTTP_200_OK
        )
    
class MyBookedEvents(APIView):
    permission_classes = [IsAuthenticated]

    def get(self, request):
        # Fetch all bookings where the current user is the booker
        bookings = Booking.objects.filter(user=request.user).select_related('event')

        # Structure response to include event details + QR code from the Booking model
        events_with_qr = [
            {
                "event": EventSerializer(booking.event).data,  # Serialize event details
                "qr_code": request.build_absolute_uri(booking.qr_code.url) if booking.qr_code else None
            }
            for booking in bookings
        ]

        return Response({"success": True, "data": events_with_qr}, status=200)